From d767361c791ad5fbed68d0e94d17aea9cd767a38 Mon Sep 17 00:00:00 2001
From: Simon Hailes <simon@screen.subtitling.com>
Date: Sun, 20 Sep 2009 11:51:52 +0100
Subject: [PATCH 1/2] dvbsub:- fixes DDS(HD res subs) malloc/free issues Todo - DDS windowed testing

---
 modules/codec/dvbsub.c |  154 ++++++++++++++++++++++++++++++-----------------
 1 files changed, 98 insertions(+), 56 deletions(-)

diff --git a/modules/codec/dvbsub.c b/modules/codec/dvbsub.c
index 8e90df8..fea080c 100644
--- a/modules/codec/dvbsub.c
+++ b/modules/codec/dvbsub.c
@@ -234,7 +234,8 @@ struct decoder_sys_t
     dvbsub_page_t   *p_page;
     dvbsub_region_t *p_regions;
     dvbsub_clut_t   *p_cluts;
-    dvbsub_display_t *p_display;
+    /* this is very small, so keep forever */
+    dvbsub_display_t display;    
     dvbsub_clut_t    default_clut;
 };
 
@@ -279,6 +280,7 @@ static void decode_clut( decoder_t *, bs_t * );
 static void free_all( decoder_t * );
 
 static void default_clut_init( decoder_t * );
+static void default_dds_init( decoder_t * );
 
 static subpicture_t *render( decoder_t * );
 
@@ -313,7 +315,9 @@ static int Open( vlc_object_t *p_this )
     p_sys->p_regions      = NULL;
     p_sys->p_cluts        = NULL;
     p_sys->p_page         = NULL;
-    p_sys->p_display      = NULL;
+
+    /* configure for SD res in case DDS is not present */
+    default_dds_init( p_dec );
 
     var_Create( p_this, DVBSUB_CFG_PREFIX "position",
                 VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
@@ -376,6 +380,11 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
     p_block = *pp_block;
     *pp_block = NULL;
 
+    /* configure for SD res in case DDS is not present */
+    /* a change of PTS is a good indication we must get a new DDS */
+    if (p_sys->i_pts != p_block->i_pts)
+        default_dds_init( p_dec );
+ 
     p_sys->i_pts = p_block->i_pts;
     if( p_sys->i_pts <= 0 )
     {
@@ -952,63 +961,68 @@ static void decode_display_definition( decoder_t *p_dec, bs_t *s )
     decoder_sys_t *p_sys = p_dec->p_sys;
     uint16_t      i_segment_length;
     uint16_t      i_processed_length = 40;
-    dvbsub_display_t *p_display;
-    dvbsub_display_t *p_old = p_sys->p_display;
     int           i_version;
 
     i_segment_length = bs_read( s, 16 );
     i_version        = bs_read( s, 4 );
 
     /* Check version number */
-    if( p_old && ( p_old->i_version == i_version ) )
+    if( p_sys->display.i_version == i_version )
     {
         /* The definition did not change */
-        bs_skip( s, 8*i_segment_length - 4 );
-        return;
-    }
-
+        /* but we'll read it anyway */
 #ifdef DEBUG_DVBSUB
-    msg_Dbg( p_dec, "new display definition: %i", i_version );
+        msg_Dbg( p_dec, "repeated display definition: %i", i_version );
 #endif
-    p_display = malloc( sizeof(dvbsub_display_t) );
-    if( p_display )
+    }
+    else
     {
-        /* We don't have this version of the display definition: Parse it */
-        p_display->i_version = i_version;
-        p_display->b_windowed = bs_read( s, 1 );
-        bs_skip( s, 3 ); /* Reserved bits */
-        p_display->i_width = bs_read( s, 16 )+1;
-        p_display->i_height = bs_read( s, 16 )+1;
-
-        if( p_display->b_windowed )
-        {
 #ifdef DEBUG_DVBSUB
-            msg_Dbg( p_dec, "display definition with offsets (windowed)" );
+        msg_Dbg( p_dec, "new display definition: %i", i_version );
 #endif
-            /* Coordinates are measured from the top left corner */
-            p_display->i_x     = bs_read( s, 16 );
-            p_display->i_max_x = bs_read( s, 16 );
-            p_display->i_y     = bs_read( s, 16 );
-            p_display->i_max_y = bs_read( s, 16 );
-            i_processed_length += 64;
-        }
     }
 
-    p_sys->p_display = p_display;
-    free( p_old );
+    /* Parse the display definition: */
+    p_sys->display.i_version = i_version;
+    p_sys->display.b_windowed = bs_read( s, 1 );
+    bs_skip( s, 3 ); /* Reserved bits */
+    p_sys->display.i_width = bs_read( s, 16 )+1;
+    p_sys->display.i_height = bs_read( s, 16 )+1;
+
+    if( p_sys->display.b_windowed )
+    {
+#ifdef DEBUG_DVBSUB
+        msg_Dbg( p_dec, "display definition with offsets (windowed)" );
+#endif
+        /* Coordinates are measured from the top left corner */
+        p_sys->display.i_x     = bs_read( s, 16 );
+        p_sys->display.i_max_x = bs_read( s, 16 );
+        p_sys->display.i_y     = bs_read( s, 16 );
+        p_sys->display.i_max_y = bs_read( s, 16 );
+        i_processed_length += 64;
+    }
+    else
+    {
+        /* if not windowed, setup the window variables to good defaults */
+        /* not necessary, but.. */
+        p_sys->display.i_x     = 0;
+        p_sys->display.i_max_x = p_sys->display.i_width-1;
+        p_sys->display.i_y     = 0;
+        p_sys->display.i_max_y = p_sys->display.i_height-1;
+    }
 
     if( i_processed_length != i_segment_length*8 )
     {
-        msg_Err( p_dec, "processed length %d != segment length %d",
-                 i_processed_length, i_segment_length );
+        msg_Err( p_dec, "processed length %d bytes != segment length %d bytes",
+                 i_processed_length/8, i_segment_length );
     }
 
 #ifdef DEBUG_DVBSUB
     msg_Dbg( p_dec, "version: %d, width: %d, height: %d",
-             p_display->i_version, p_display->i_width, p_display->i_height );
-    if( p_display->b_windowed )
+             p_sys->display.i_version, p_sys->display.i_width, p_sys->display.i_height );
+    if( p_sys->display.b_windowed )
         msg_Dbg( p_dec, "xmin: %d, xmax: %d, ymin: %d, ymax: %d",
-                 p_display->i_x, p_display->i_max_x, p_display->i_y, p_display->i_max_y );
+                 p_sys->display.i_x, p_sys->display.i_max_x, p_sys->display.i_y, p_sys->display.i_max_y );
 #endif
 }
 
@@ -1409,7 +1423,6 @@ static void free_all( decoder_t *p_dec )
     dvbsub_region_t *p_reg, *p_reg_next;
     dvbsub_clut_t *p_clut, *p_clut_next;
 
-    free( p_sys->p_display );
 
     for( p_clut = p_sys->p_cluts; p_clut != NULL; p_clut = p_clut_next )
     {
@@ -1468,19 +1481,19 @@ static subpicture_t *render( decoder_t *p_dec )
     p_spu->i_original_picture_width = 720;
     p_spu->i_original_picture_height = 576;
 
-    if( p_sys->p_display )
-    {
-        p_spu->i_original_picture_width = p_sys->p_display->i_width;
-        p_spu->i_original_picture_height = p_sys->p_display->i_height;
+    p_spu->i_original_picture_width = p_sys->display.i_width;
+    p_spu->i_original_picture_height = p_sys->display.i_height;
 
-        if( p_sys->p_display->b_windowed )
-        {
-            /* TODO: check that this actually works */
-            p_spu->i_original_picture_width = p_sys->p_display->i_max_x - p_sys->p_display->i_x;
-            p_spu->i_original_picture_height = p_sys->p_display->i_max_y - p_sys->p_display->i_y;
-            i_base_x += p_sys->p_display->i_x;
-            i_base_y += p_sys->p_display->i_y;
-        }
+    if( p_sys->display.b_windowed )
+    {
+	/* From en_300743v01 - */
+	/* the DDS is there to indicate intended size/position of text */
+	/* the intended video area is ->i_width/height */
+        /* the window is within this... SO... we should leave i_original_picture_width etc. as is */
+        /* and ONLY change i_base_x.  effectively i_max_x/y are only there to limit memory requirements*/
+        /* we COULD use the upper limits to limit rendering to within these? */
+        i_base_x += p_sys->display.i_x;
+        i_base_y += p_sys->display.i_y;
     }
 
     pp_spu_region = &p_spu->p_region;
@@ -1507,10 +1520,6 @@ static subpicture_t *render( decoder_t *p_dec )
 
         p_regiondef = &p_sys->p_page->p_region_defs[i];
 
-#ifdef DEBUG_DVBSUB
-        msg_Dbg( p_dec, "rendering region %i (%i,%i)", i,
-                 p_regiondef->i_x, p_regiondef->i_y );
-#endif
 
         /* Find associated region */
         for( p_region = p_sys->p_regions; p_region != NULL;
@@ -1519,6 +1528,22 @@ static subpicture_t *render( decoder_t *p_dec )
             if( p_regiondef->i_id == p_region->i_id ) break;
         }
 
+#ifdef DEBUG_DVBSUB
+	/* if a region exists, then print it's size */
+	if (p_region)
+	{
+	        msg_Dbg( p_dec, "rendering region %i (%i,%i) to (%i,%i)", i,
+                 	p_regiondef->i_x, p_regiondef->i_y, 
+			p_regiondef->i_x + p_region->i_width, 
+			p_regiondef->i_y + p_region->i_height );
+	}
+	else
+	{
+        	msg_Dbg( p_dec, "rendering region %i (%i,%i) (no region matched to render)", i,
+                 p_regiondef->i_x, p_regiondef->i_y );
+	}
+#endif
+
         if( !p_region )
         {
             msg_Dbg( p_dec, "region %i not found", p_regiondef->i_id );
@@ -1718,7 +1743,8 @@ static subpicture_t *YuvaYuvp( subpicture_t *p_subpic )
                         + p_region->p_picture->p[0].i_pitch * 1 / 3;
         int i_tolerance = 0;
 
-#ifdef DEBUG_DVBSUB
+/* p_enc not valid here */
+#ifdef DEBUG_DVBSUB1
         msg_Dbg( p_enc, "YuvaYuvp: i_pixels=%d, i_iterator=%d", i_pixels, i_iterator );
 #endif
         p_fmt->i_chroma = VLC_CODEC_YUVP;
@@ -1795,7 +1821,8 @@ static subpicture_t *YuvaYuvp( subpicture_t *p_subpic )
             }
         }
 
-#ifdef DEBUG_DVBSUB
+/* p_enc not valid here */
+#ifdef DEBUG_DVBSUB1
         msg_Dbg( p_enc, "best palette has %d colors", p_fmt->p_palette->i_entries );
 #endif
 
@@ -1889,7 +1916,9 @@ static subpicture_t *YuvaYuvp( subpicture_t *p_subpic )
             p_fmt->p_palette->palette[i][3] = 0;
         }
         p_fmt->p_palette->i_entries = i_max_entries;
-#ifdef DEBUG_DVBSUB
+
+/* p_enc not valid here */
+#ifdef DEBUG_DVBSUB1
         msg_Dbg( p_enc, "best palette has %d colors", p_fmt->p_palette->i_entries );
 #endif
     }
@@ -2608,3 +2637,16 @@ static void encode_pixel_line_8bp( bs_t *s, subpicture_region_t *p_region,
     /* Stuffing */
     bs_align_0( s );
 }
+
+
+static void default_dds_init( decoder_t * p_dec )
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+
+    /* configure for SD res in case DDS is not present */
+    p_sys->display.i_version = -1;
+    p_sys->display.i_width = 720;
+    p_sys->display.i_height = 576;
+    p_sys->display.b_windowed = false;
+}
+
-- 
1.6.4.msysgit.0

